%% Produce histogram of a vector/matrix, with an easy setting of from, to and
%% logarithmic spacing.
%% If available, it calls histogramC -- fast C++ implementation of the histogram
%% function, otherwise slow histogramM implementation.
%%
%% Usage:
%%	[ [x, ] h ] = histogram ( image [, bins [, zfrom, zto] [, do_log ]] )
%%	[x, h ] = histogram ( image );
%%	h = histogram ( image, bins );
%%	h = histogram ( image, bins, do_log );
%%	plot ( histogram ( image, bins, zfrom, zto ) );
%%	histogram ( image, bins, zfrom, zto, do_log )
%%
%% For do_log = 0, bins are equidistant in linear space.
%% For do_log != 0, bins are equidistant in logarithmic space.
%%
%% When using linear space, then this routine is equivalent to this call:
%%	hist(a, linspace(zfrom, zto, bins))
%% For logarithmic spacing, it is faster (mainly for large images) than
%% calculating temporary data of log(data). Notice that it does not matter on
%% base for the logarithm -- only ratios of histogram bins matter.
%%
%% Notice the reversed order of output arguments x,h compared to hist.m.
%%
%% Note: You can plot histogram with bar(x,h) or plot(x,y) or stairs(x,y) or
%% histDrawPm3d(x,h).
%%
%% Example: [x,h] = histogram(a,100,1,100);     bar(x,h);
%% Example: [x,h] = histogram(a,100,3,3e6,1);   histDrawPm3d(x,h);
%% Example: [x,h] = histogram(a,100,2,2**16,1); stairs(x,h);
%%
%% Author and copyright: Petr Mikulik
%%
%% History: May 2004 -- change this routine to choose whether to call C++ or M
%%			implementation of histogram.
%%
%% License: You can redistribute this code and/or modify it under the terms of
%%	    the GNU General Public License as published by the Free Software
%%	    Foundation; either version 2, or (at your option) any later version.
%%

% function [ z, h ] = histogram ( a, bins, zfrom, zto, do_log )
function [ z, h ] = histogram ( a, varargin )

if nargin == 0 | nargin > 5
  fprintf('Usage:  histogram ( data [, options ] )\n');
  return;
end

% Choose which implementation of histogram to call: C++ or M
if exist('histogramC')
    histName='histogramC';
else
    histName='histogramM';
end

% Make the command like [z,h]=histogramX(a,varargin{1},varargin{2},..)
cmd = sprintf(',varargin{%i}', 1:length(varargin));
cmd=['[z,h]=', histName, '(a', cmd, ');'];
eval(cmd);


% endfunction

% eof histogram.m
