 /******************** histogramC.cpp *********************
 *
 * Author: Petr Mikulik
 * (c) Petr Mikulik, Institute of Condensed Matter Physics,
 *     Masaryk University, Brno
 *     April 2004
 *
 * Calculation of histogram for Octave, written in C++.
 *
 * License: GNU GPL
 *
 */


#include <oct.h>
#include "pm_hist.h"

//#define HIST_VERBOSE 1

DEFUN_DLD (histogramC, args, nargout,
"\
Produce histogram of a vector/matrix, with an easy setting of from, to and\n\
logbase spacing.\n\
It is like my histogram.m, but implemented in C++ and thus very fast.\n\
\n\
Usage:\n\
        [ [x, ] h ] = histogramC ( image [, bins [, zfrom, zto] [, logbase ]] )\n\
        [x, h ] = histogramC ( image );\n\
        h = histogramC ( image, bins );\n\
        h = histogramC ( image, bins, logbase );\n\
        plot ( histogramC ( image, bins, zfrom, zto ) );\n\
        histogramC ( image, bins, zfrom, zto, logbase )\n\
\n\
Note 1: logbase set to 0 is equivalent to linear, and if set to 1 then it is\n\
	replaced by 10 (to be compatible with original want_logbase_10).\n\
Note 2: you can plot histogram with bar(x,h) or plot(x,y) or stairs(x,y) or\n\
histDrawPm3d(x,h).\n\
\n\
Example: [x,h] = histogramC(a, 100, 3, 3e6, 1);\n\
         plot(x,h);\n\
\n\
Author: Petr Mikulik\n\
Version: April 2004\n\
")
{
    octave_value_list retval = octave_value(1);

    int nargin = args.length();
    int k;

#ifdef HIST_VERBOSE
    octave_stdout <<"histogramC: nargin=" <<nargin <<" nargout=" <<nargout <<"\n";
#endif

    if (nargin < 1 || nargin > 5) {
	error("histogramC() requires 1 upto 5 arguments");
	print_usage ("histogramC");
	return retval;
    }
    if (!(args(0).is_real_matrix() || args(0).is_range())) { // 
	error ("first argument must be real vector or matrix");
	return retval;
    }

    t_pm_histogram opts;
    opts.bins = 10;
    opts.autoscale = 1;
    opts.logbase = 0;

    if (nargin >= 2) {
	int res = args(1).is_scalar_type();
	if (res) opts.bins = args(1).int_value();
	if (!res || opts.bins <= 0) {
	    error ("number of bins must be positive integer");
	    return retval;
	}
    }
#ifdef HIST_VERBOSE
    octave_stdout <<"histogramC: bins = " <<opts.bins <<"\n";
#endif

    if (nargin >= 4) {
	int res = (args(2).is_scalar_type() && args(3).is_scalar_type());
	if (!res) {
	    error ("zfrom and zto must be scalars");
	    return retval;
	}
	opts.autoscale = 0;
	opts.zfrom = args(2).double_value();
	opts.zto   = args(3).double_value();
    }

    if (nargin == 3 || nargin == 5) {
	int res = args(nargin-1).is_scalar_type();
	if (res) opts.logbase = args(nargin-1).double_value();
	if (!res || opts.logbase < 0) {
	    error("logbase cannot be negative");
	    return retval;
	}
	if (opts.logbase == 1.0) opts.logbase = 10;
    }
#ifdef HIST_VERBOSE
    octave_stdout <<"histogramC: logbase = " <<opts.logbase <<"\n";
#endif

    int n = args(0).columns() * args(0).rows();
    const double *a = args(0).matrix_value().data();
    // The above a is OK for vectors and matrices, but it fails for range
    // in Octave 2.1.49 (obviously a bug), thus we have to copy the data:
    double *atmp = NULL;
    if (args(0).is_range()) {
	a = atmp = new double [n];
	Matrix m = args(0).range_value().matrix_value();
	for (k = 0; k < n; k++)
	    atmp[k] = m(0, k);
    }

#ifdef HIST_VERBOSE
    octave_stdout <<"c = " <<args(0).columns() <<"  r = " <<args(0).rows() <<"\n";
    if (args(0).is_range()) octave_stdout <<"IT IS RANGE\n";
    for (k = 0; k < n; k++)
	octave_stdout <<"\t" <<a[k] <<"\n";
#endif
    double *x = new double [opts.bins];
    unsigned long *h = new unsigned long [opts.bins];

    int res = pm_histogram ( x, h, a, n, opts );

#ifdef HIST_VERBOSE
    for (k = 0; k < opts.bins; k++)
	octave_stdout <<"\t here " <<x[k] <<"  \t" <<h[k] <<"\n";
#endif

    Matrix matrix_x(1, opts.bins);
    Matrix matrix_h(1, opts.bins);
    for (k = 0; k < opts.bins; k++) {
	matrix_x(0,k) = x[k];
	matrix_h(0,k) = h[k];
    }

    delete [] x;
    delete [] h;
    delete [] atmp;

    if (nargout <= 1)
	retval = octave_value(matrix_h);
    else {
	retval(0) = octave_value(matrix_x);
	retval(1) = octave_value(matrix_h);
    }

    return retval;
}

// histogramC.cpp
