%% Produce histogram of a vector/matrix, with an easy setting of from, to and
%% logarithmic spacing.
%%
%% Usage:
%%	[ [x, ] h ] = histogram ( image [, bins [, zfrom, zto] [, do_log ]] )
%%	[x, h ] = histogram ( image );
%%	h = histogram ( image, bins );
%%	h = histogram ( image, bins, do_log );
%%	plot ( histogram ( image, bins, zfrom, zto ) );
%%	histogram ( image, bins, zfrom, zto, do_log )
%%
%% For do_log = 0, bins are equidistant in linear space.
%% For do_log != 0, bins are equidistant in logarithmic space.
%%
%% When using linear space, then this routine is equivalent to this call:
%%	hist(a, linspace(zfrom, zto, bins))
%% For logarithmic spacing, it is faster (mainly for large images) than
%% calculating temporary data of log(data). Notice that it does not matter on
%% base for the logarithm -- only ratios of histogram bins matter.
%%
%% Notice the reversed order of output arguments x,h compared to hist.m.
%%
%% Note: You can plot histogram with bar(x,h) or plot(x,y) or stairs(x,y) or
%% histDrawPm3d(x,h).
%%
%% Example: [x,h] = histogram(a,100,1,100);     bar(x,h);
%% Example: [x,h] = histogram(a,100,3,3e6,1);   histDrawPm3d(x,h);
%% Example: [x,h] = histogram(a,100,2,2**16,1); stairs(x,h);
%%
%% Author and copyright: Petr Mikulik
%%
%% History: 18. 4. 2004 -- use Octave's hist.m algorithm for bins >= 30
%%	    31. 3. 2000 -- original version
%%
%% License: You can redistribute this code and/or modify it under the terms of
%%	    the GNU General Public License as published by the Free Software
%%	    Foundation; either version 2, or (at your option) any later version.
%%

function [ z, h ] = histogramM ( a, bins, zfrom, zto, do_log )

if nargin == 0 | nargin > 5
  fprintf('Usage:  histogram ( data [, options ] )\n');
  return;
end

a = a(:); % convert matrix to a vector (if not already a vector)

if nargin == 1 % histogram(a)
  bins = 100;
  do_log = 0;
  zfrom = min(a); zto = max(a);
elseif nargin == 2 % histogram (a, bins)
  do_log = 0;
  zfrom = min(a); zto = max(a);
elseif nargin == 3 % histogram (a, bins, do_log)
  do_log = zfrom;
  zfrom = min(a); zto = max(a);
elseif nargin == 4 % histogram (a, bins, zfrom, zto)
  do_log = 0;
elseif nargin == 5 % histogram (a, bins, zfrom, zto, do_log)
end

if do_log & zfrom<=0 % find minimum for positive elements
  % remove non-positive elements
  zfrom = min( a(a>0) );
  if length(zfrom) == 0
    fprintf('Error: no data element positive\n');
    return
  end
end

fprintf('Histogram:  bins=%i  from=%g  to=%g  ', bins, zfrom, zto);
if (do_log)
    fprintf('logarithmic\n');
else
    fprintf('linear\n');
end

% Make nodes; bins are in between them:
if (do_log)
  % Don't use logspace() as below because it gets strange for zto=pi:
  % z = logspace(log10(zfrom),log10(zto),bins);
  z = exp (linspace(log(zfrom), log(zto), bins));
  % cutoff = boundaries in between bins are equidistant in logarithmic
  % space => geometric mean
  cutoff = sqrt( z(1:end-1) .* z(2:end) );
else
  z = linspace(zfrom,zto,bins);
  % cutoff = boundaries in between bins are equidistant in linear 
  % space => arithmetic mean
  cutoff = (z(1:end-1) + z(2:end)) * 0.5;
end

% The code below is according to Octave script plot/hist.m; however, it differs
% in the cutoff definition for for logarithmic bins which function is not
% available in the traditional histogram:
n = length (z);
if (n < 30)
  % The following algorithm works fastest for n less than about 30.
  chist = [zeros(n,1); length(a)];
  for i = 1:n-1
    chist(i+1) = sum (a < cutoff(i));
  end
else
  % The following algorithm works fastest for n greater than about 30.
  % Put cutoff elements between boundaries, integrate over all
  % elements, keep totals at boundaries.
  [s, idx] = sort ([cutoff(:); a(:)]);
  chist = cumsum(idx>=n);
  chist = [0; chist(idx<n); chist(end)];
end
h = diff(chist)';

% endfunction

% eof histogramM.m
